/*
 * ana_exec.c
 *
 * Date         : 29/10/2001
 * Author       : Akihiro Asahara (asahara@cr.scphys.kyoto-u.ac.jp)
 * version      : 0.9
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "cfortran.h"
#include "hbook.h"
#include "chess_local.h"
#include "read_module.h"

void ana_exec(int *idata, int *iheader)
{
  int i,j,m,ch,x,y;

  struct vme_time vtime;
  int daqflag;
  struct gps_time gtime;

  static int prescal[9]= {0,0,0,0,0,0,0,0,0};
  int scal_c[9],scal_r[9];
  short scalflag;
  int dev;
  double result[10],gps_min;
  int tmp_data;

  int tko[46];
  short tkoflag=0;
  static int tko_i[9][9];

  int adc[3];
  int sum_adc;
  short adcflag;

  struct tdcdata tdc[256];
  short tdcflag;
  int tdc_length=0;
  int nhit;
  int pmtnum[46]; 
  int start_i[9][9],stop_i[9][9],width_i[9][9];
  int tmp;

 /* event length */

  HF1(102,(float)iheader[0],1.0);

 /* Sytem time  */

  systime_read(idata,&vtime);

 /* Trigger flag (GPS ? or event ?) */

  daqflag = idata[3] & 0x3;
  HF1(101,(float)daqflag,1.0);

 /* GPS  */

  gps_read(idata,&gtime);

 /* SCAL */

  scal_read(idata,scal_r,scal_c);

  dev = scal_c[GPS] - prescal[GPS];

  if(dev < 0){
    printf("error! : scal\n");
    printf("now: %d  != pre: %d \n",scal_c[GPS],prescal[GPS]);
    scalflag =2;
  }
  else if(0<= dev && dev < UPDATE){
    /* not update */
    scalflag =0;
  }
  else{
    /* update */
    scalflag =1;
    for(i=0;i<SOFT_SCAL_NUM;i++){
      
      tmp_data = scal_c[i*2] - prescal[i*2];
      gps_min = (unsigned)scal_c[GPS]/60.0;
      
      if(tmp_data <0){
	tmp_data = ~tmp_data + 1;
	result[i] = (unsigned)tmp_data/(float)UPDATE/2.0;
      }
      else result[i] = (unsigned)tmp_data/(float)UPDATE/2.0;
      
    }
    for(i=SOFT_SCAL_NUM;i<5;i++){
      result[i] = (unsigned)(scal_c[i*2] - prescal[i*2])/(float)UPDATE/2.0;
    }
	
	HF1(41,(float)result[1],gps_min);
	HF1(43,(float)result[3],gps_min);
	HF1(44,(float)result[4],gps_min);
	HF1(45,(float)result[5],gps_min);
    for(i=0;i<9;i++) prescal[i] = scal_c[i];
  }


  HF1(50,(float)scalflag,1.0);
  
 /* TKO scal */

  tkoflag = tko_read(idata,tko);
  HF1(30,(float)tkoflag,1.0);

  if(tkoflag){
    for(ch=1;ch<=PMT_NUM;ch++){
      tko_i[xpos[ch]][ypos[ch]] = tko[ch];
      HF2(21,(float)ch,(float)tko[ch],1.0);
    }
    for(x=1;x<=7;x++){
      for(y=1;y<=7;y++){
        HF2(22,(float)x,(float)y,(float)tko_i[x][y]);
      }
    }
  }

  if(daqflag ==2)  return;

 /* ADC */

  adcflag = adc_read(idata,adc);
  HF1(70,(float)adcflag,1.0);

  sum_adc = adc[0] + adc[1] + adc[2];

  if(adcflag==0){
    HF1(61,(float)adc[0],1.0);
    HF1(62,(float)adc[1],1.0);
    HF1(63,(float)adc[2],1.0);
    HF1(64,(float)sum_adc,1.0);
  }
  
 /*
  * TDC
  */

  tdcflag = tdc_read(idata,tdc,&tdc_length);
  HF1(10,(float)tdcflag,1.0);
  
  if(tdcflag !=0) return;

  /* Initialize */

  nhit =0;

  for(i=0;i<9;i++){
    for(j=0;j<9;j++){
      start_i[i][j] = 0;
      stop_i[i][j] = 0;
      width_i[i][j] = 0;
    }
  }

  /* Store start,stop,width */

  for(i=0;i<(tdc_length-1);i++){

    HF1(6,(float)tdc[i].channel,1.0);

    if(tdc[i].channel != BAD_CH && tdc[i].edge == UP){
      
      /* leading edge */
      
      nhit += 1;
      HF2(2,(float)tdc[i].channel,(float)tdc[i].time,1.0);
      start_i[xpos[tdc[i].channel]][ypos[tdc[i].channel]]
	  = tdc[i].time;
      
      for(j=i+1;j < (tdc_length-1) ;j++){
		if(   tdc[j].channel==tdc[i].channel
			  && tdc[j].edge == DOWN){
		  
          HF2(3,(float)tdc[i].channel,(float)tdc[i].time,1.0);
		  stop_i[xpos[tdc[i].channel]][ypos[tdc[i].channel]]
			= tdc[j].time;
		  
          tmp_data = tdc[j].time - tdc[i].time;
          HF2(1,(float)tdc[i].channel,(float)tmp_data,1.0);
		  width_i[xpos[tdc[i].channel]][ypos[tdc[i].channel]]
			= tmp_data;
		  break;
		}
      }
    }
  }
  HF1(5,(float)nhit,1.0);

  for(x=1;x<=7;x++){
    for(y=1;y<=7;y++){
      HF2(7,(float)x,(float)y,(float)width_i[x][y]);
      HF2(8,(float)x,(float)y,(float)start_i[x][y]);
      HF2(9,(float)x,(float)y,(float)stop_i[x][y]);
    }
  }

#if 0  
  dump(tko_i);
  dump(start_i);
  dump(stop_i);
  dump(width_i);
#endif

  /*
   * Scan multi hit
   */

  for(i=0;i<46;i++) pmtnum[i] = 0;

  for(ch=1;ch<=45;ch++){
    for(i=0;i < (tdc_length-1) ;i++){
      if(tdc[i].channel == ch && tdc[i].edge){
	pmtnum[ch] +=1;
      }
    }
    HF2(4,(float)ch,(float)pmtnum[ch],1.0);
  }

  return;
}
