/*
 * read_module.c
 */

#include <stdio.h>
#include <time.h>
#include "chess_local.h"

#define BAD_CH 60

void systime_read(int *idata, struct vme_time *vtime)
{
  int tmp;

  tmp = idata[0];
  
  vtime->hh = (tmp >>12) & 0x3f;
  vtime->ml = (tmp >>6)  & 0x3f;
  vtime->ss = tmp & 0x3f;

  vtime->tv_sec  = idata[1];
  vtime->tv_usec = idata[2];
  
  return;
}

void gps_read(int *idata, struct gps_time *gtime)
{
  short *gps;

  gps = (short *)(idata + 4);

  gtime->yy = gps[0] >>8;
  gtime->mm = gps[0] & 0xff;
  gtime->dd = gps[1] >>8;
  gtime->hh = gps[1] & 0xff;
  gtime->ml = gps[2] >>8;
  gtime->ss = gps[2] & 0xff;
  gtime->msec = (unsigned short)gps[3];
  gtime->l_usec = gps[4];
  gtime->h_usec = gps[5]; 
  
  return;
}

void scal_read(int *idata, int *scal_r, int *scal_c)
{
  int i;

  for(i=0;i<8;i++){
    scal_r[i] = idata[7+i];
  }
  for(i=0;i<9;i++){
    scal_c[i] = idata[15+i];
  }

  return;
}

short tko_read(int *idata, int *tko)
{
  int i,ch;
  short *tko_data;
  short tkoflag;

  tkoflag = idata[25];

  if(tkoflag == 0) return tkoflag;

  tko_data = (short *)(idata + 26);
  
  for(i=0;i<64;i++){

    if(16<=i&&i<=30){
      ch = i+15;
      tko[ch] = tko_data[i];
    }
    else if(32<=i&&i<=39){
      ch = i-17;
      tko[ch] = tko_data[i];
    }
    else if(40<=i&&i<=46){
      ch = i-16;
      tko[ch] = tko_data[i];
    }
    else if(48<=i&&i<=61){
      ch = i -47;
      tko[ch] = tko_data[i];
    }
  }
  /* center PMT */
  tko[23] = 0;
  
  return tkoflag;
}

short adc_read(int *idata, int *adc)
{
  int i;
  short adcflag=0;

  for(i=0;i<=2;i++){
    adc[i] = idata[58+i];
    if(adc[i] < 0) adcflag += 1;
  }

  return adcflag;
}

short tdc_read(int *idata, struct tdcdata *tdc, int *tdc_length)
{
  int tmp_data;
  int header_f;
  int counter;
  int i;
  short tdcflag=0;

  *tdc_length = idata[61];

  if(*tdc_length<=1){ /* TDC is empty */
    tdcflag =1;
    return tdcflag;
  }

  /* header */
  
  tmp_data = idata[62];
  
  header_f = tmp_data >> 29 & 0x3;
  if(header_f != 0x3){
    printf("error: in TDC header flag\n");
    tdcflag =2;
    return tdcflag;
  }
  counter = tmp_data & 0xffff;

  /* data transrate */

  
  for(i=0;i < (*tdc_length-1) ;i++){
    tmp_data = idata[i+63];

    header_f = tmp_data >> 29 & 0x3;
    if(header_f == 0x3){
      printf("error: in TDC data\n");
      printf("data = %x lenght = %d\n",tmp_data,*tdc_length);
      tdcflag =3;
      return tdcflag;
    }

    tdc[i].channel = tmp_data >> 22 & 0x3f;
    tdc[i].chip_n  = tmp_data >> 28 & 0x1;
    tdc[i].edge    = tmp_data>>21 & 0x1;
    tdc[i].time    = tmp_data & 0x1fffff;
    tdc[i].channel = tdc[i].channel + (tdc[i].chip_n * 32);
    
    /* 
     *  Set channel#
     */

    if(tdc[i].channel >= 16 && tdc[i].channel <= 29){
      tdc[i].channel -= 15;
    }
    else if(tdc[i].channel >= 32 && tdc[i].channel <= 39){
      tdc[i].channel -= 17;
    }
    else if(tdc[i].channel >= 40 && tdc[i].channel <= 46){
      tdc[i].channel -= 16;
    }
    else if(tdc[i].channel >= 48 && tdc[i].channel <= 62){
      tdc[i].channel -= 17;
    }
    else {
      tdc[i].channel = BAD_CH;
    }
  }

  return tdcflag;
}

void dump(int image[][9])
{

  int y;

  printf("+-----------------------------------+\n");
  for(y=7;y>=1;y--){
    if(y==1 || y==7){
      printf("|     %4d %4d %4d %4d %4d      | \n"
	     ,image[2][y],image[3][y],image[4][y]
	     ,image[5][y],image[6][y]);
    }
    else{
      printf("|%4d %4d %4d %4d %4d %4d %4d | \n"
	     ,image[1][y],image[2][y],image[3][y],image[4][y]
	     ,image[5][y],image[6][y],image[7][y]);
    }
  }
  printf("+-----------------------------------+\n");
  return;
}  

